/* vim: set et sw=2: */

#include "gpager.h"

void
value_changed_notify (GConfClient *client,
                      guint        cnxn_id,
                      GConfEntry  *entry,
                      gpointer     user_data)
{
  int value = 0;
  gboolean bvalue = FALSE;
  gchar* key;
  GpagerConfig *config = user_data;
  GConfValue* gconfvalue = gconf_entry_get_value (entry);
  
  if (gconfvalue != NULL) {
    if (gconfvalue->type == GCONF_VALUE_INT) {
    value = gconf_value_get_int (gconfvalue);
    }
    else if (gconfvalue->type == GCONF_VALUE_BOOL) {
    bvalue = gconf_value_get_bool (gconfvalue);
    }
    else {
      g_warning("Wrong type: %s", entry->key);
    return;
    }
  }
  else {
    g_warning("Null value: %s", entry->key);
    return;
  }

  key = g_path_get_basename (entry->key);

  /* do the right thing wrt the value that was changed */
  if (g_str_equal(key, "width")) {
    config->width = value;
    gpager_redraw ();
  }
  else if (g_str_equal(key, "height")) {
    config->height = value;
    gpager_redraw ();
  }
  else if (g_str_equal(key, "num_rows")) {
    config->rows = value;
    gpager_update_geometry ();
    gpager_redraw ();
  }
  else if (g_str_equal(key, "dynamic_rows")) {
    config->dynamic_rows = bvalue;
    gpager_update_geometry ();
    gpager_redraw ();
  }
  else if (g_str_equal(key, "border_width")) {
    gpager_set_border_width (value);
  }
  else if (g_str_equal(key, "keep_above")) {
    printf("keep_above: %i\n", bvalue);
    gpager_keep_above (bvalue);
  }
  else if (g_str_equal(key, "num_workspaces")) {
    config->workspaces = value;
    gpager_update_geometry ();
    gpager_redraw ();
  }
}

gint
get_configurable_int (GConfClient *client,
                      gchar  *key,
                      gpointer user_data)
{
  gconf_client_notify_add (client, key,
                           value_changed_notify,
                           user_data, NULL, NULL);
  return gconf_client_get_int (client, key, NULL);
}

gboolean
get_configurable_bool (GConfClient *client,
                       gchar  *key,
                       gpointer user_data)
{
  gconf_client_notify_add (client, key,
                           value_changed_notify,
                           user_data, NULL, NULL);
  return gconf_client_get_bool (client, key, NULL);
}

void
load_config (GConfClient *client, GpagerConfig *config)
{
  config->workspaces = get_configurable_int (client,
      "/apps/metacity/general/num_workspaces", config);
  config->rows = get_configurable_int (client,
      "/apps/gpager/num_rows", config);
  config->pos_x = get_configurable_int (client,
      "/apps/gpager/pos_x", config);
  config->pos_y = get_configurable_int (client,
      "/apps/gpager/pos_y", config);
  config->width = get_configurable_int (client,
      "/apps/gpager/width", config);
  config->height = get_configurable_int (client,
      "/apps/gpager/height", config);
  config->border_width = get_configurable_int (client,
      "/apps/gpager/border_width", config);
  config->dynamic_rows = get_configurable_bool (client,
      "/apps/gpager/dynamic_rows", config);
  config->keep_above = get_configurable_bool (client,
      "/apps/gpager/keep_above", config);
}

void
save_position (GtkWindow *win)
{
  gint x, y;
  GConfClient *client;

  client = g_object_get_data (G_OBJECT (win), "client");
  gtk_window_get_position (win, &x, &y);
  gconf_client_set_int (client, "/apps/gpager/pos_x", x, NULL);
  gconf_client_set_int (client, "/apps/gpager/pos_y", y, NULL);
  gtk_window_get_size (win, &x, &y);
  gconf_client_set_int (client, "/apps/gpager/width", x, NULL);
  gconf_client_set_int (client, "/apps/gpager/height", y, NULL);
}

